﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectSeaplane.Entities;

namespace ProjectSeaplane.DrawningObjects
{
    /// <summary>
    /// Расширение для класса EntityPlane
    /// </summary>
    public static class ExtentionDrawningPlane
    {
        /// <summary>
        /// Создание объекта из строки
        /// </summary>
        /// <param name="info">Строка с данными для создания объекта</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <param name="width">Ширина</param>
        /// <param name="height">Высота</param>
        /// <returns>Объект</returns>
        public static DrawningPlane? CreateDrawningPlane(this string info, char
        separatorForObject, int width, int height)
        {
            string[] strs = info.Split(separatorForObject);
            if (strs.Length == 3)
            {
                return new DrawningPlane(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]), Color.FromName(strs[2]), width, height);
            }
            if (strs.Length == 6)
            {
                return new DrawningSeaplane(Convert.ToInt32(strs[0]),
                Convert.ToInt32(strs[1]),
                Color.FromName(strs[2]),
                Color.FromName(strs[3]),
                Convert.ToBoolean(strs[4]),
                Convert.ToBoolean(strs[5]), width, height);
            }
            return null;
        }
        /// <summary>
        /// Получение данных для сохранения в файл
        /// </summary>
        /// <param name="drawningPlane">Сохраняемый объект</param>
        /// <param name="separatorForObject">Разделитель даннных</param>
        /// <returns>Строка с данными по объекту</returns>
        public static string GetDataForSave(this DrawningPlane drawningPlane,
        char separatorForObject)
        {
            var plane = drawningPlane.EntityPlane;
            if (plane == null)
            {
                return string.Empty;
            }
            var str =
            $"{plane.Speed}{separatorForObject}{plane.Weight}{separatorForObject}{plane.BodyColor.Name}";
            if (plane is not EntitySeaplane seaplane)
            {
                return str;
            }
            return
            $"{str}{separatorForObject}{seaplane.AdditionalColor.Name}{separatorForObject}{seaplane.Boat}{separatorForObject}{seaplane.Floater}";
        }
    }
}