﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ProjectSeaplane.DrawningObjects;


namespace ProjectSeaplane
{
    public partial class FormPlaneConfig : Form
    {
        Color defaultColor;
        /// <summary>
        /// Переменная-выбранная машина
        /// </summary>
        DrawningPlane? _plane = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawningPlane>? EventAddPlane;
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawningPlane> ev)
        {
            if (EventAddPlane == null)
            {
                EventAddPlane = ev;
            }
            else
            {
                EventAddPlane += ev;
            }
        }
        /// <summary>
        /// Добавление самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddPlane?.Invoke(_plane);
            Close();
        }
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormPlaneConfig()
        {
            InitializeComponent();
            defaultColor = labelColor.BackColor;
            buttonCancelObj.Click += (s, e) => Close();

            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
        }
        /// <summary>
        /// Отрисовать машину
        /// </summary>
        private void DrawPlane()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _plane?.SetPosition(5, 5);
            _plane?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name,
            DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _plane = new DrawningPlane((int)numericUpDownSpeed.Value,
                    (int)numericUpDownWeight.Value, Color.White, pictureBox.Width,
                    pictureBox.Height);
                    break;
                case "labelModifiedObject":
                    _plane = new DrawningSeaplane((int)numericUpDownSpeed.Value,
                    (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxFloater.Checked,
                    checkBoxBoat.Checked, pictureBox.Width, pictureBox.Height);
                    break;
            }
            DrawPlane();
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor,
                DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void labelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_plane == null)
                return;
            ((Label)sender).BackColor = (Color)e.Data.GetData(typeof(Color));
            switch (((Label)sender).Name)
            {
                case "labelColor":
                    _plane.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAdditionalColor":
                    if (!(_plane is DrawningSeaplane))
                        return;
                    (_plane as DrawningSeaplane).setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawPlane();
        }
    }
}
